/*
 *  GTObject.h
 *  All
 *
 *  Created by nhn on 11. 2. 2..
 *  Copyright 2011 nhn KEH. All rights reserved.
 *
 */
#pragma once

#if defined(__APPLE_CC__)
#include <libkern/OSAtomic.h>
#endif

class IGTObject
{
public:
#if defined(WIN32)
	typedef LONG ATOMIC_INT;
#else	
	typedef int32_t ATOMIC_INT;
#endif	
	
	IGTObject()
	{
		m_ref = 1;
	}
	
	virtual ~IGTObject() { }
	
	virtual int RetainObject(void)
	{
#if defined(WIN32)
		::InterlockedIncrement(&m_ref);
#elif defined(ANDROID)
		m_ref++;	// KEH_ANDROID_MATH
#else
		OSAtomicIncrement32(&m_ref);
#endif
		return m_ref;
	}
	
	virtual int ReleaseObject(void)
	{
		ATOMIC_INT nRef = 
#if defined(WIN32)
		::InterlockedDecrement(&m_ref);
#elif defined(ANDROID)
		nRef--;		// KEH_ANDROID_MATH
#else
		OSAtomicDecrement32(&m_ref);
#endif
		if(nRef == 0)
		{
			OnReleaseObject();
		}

		return nRef;
	}
	
	virtual int RetainCount(void)
	{
		return m_ref;
	}
	
protected:
	virtual void OnReleaseObject(void) = 0;
	
private:
	volatile ATOMIC_INT m_ref;
};


